# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
#: const MASK_ACCOUNT_FLAGS = 0x7;
MASK_ACCOUNT_FLAGS: int = 0x7
#: const MASK_ACCOUNT_FLAGS_V17 = 0xF;
MASK_ACCOUNT_FLAGS_V17: int = 0xF
#: const MAX_SIGNERS = 20;
MAX_SIGNERS: int = 20
#: const MASK_TRUSTLINE_FLAGS = 1;
MASK_TRUSTLINE_FLAGS: int = 1
#: const MASK_TRUSTLINE_FLAGS_V13 = 3;
MASK_TRUSTLINE_FLAGS_V13: int = 3
#: const MASK_TRUSTLINE_FLAGS_V17 = 7;
MASK_TRUSTLINE_FLAGS_V17: int = 7
#: const MASK_OFFERENTRY_FLAGS = 1;
MASK_OFFERENTRY_FLAGS: int = 1
#: const MASK_CLAIMABLE_BALANCE_FLAGS = 0x1;
MASK_CLAIMABLE_BALANCE_FLAGS: int = 0x1
#: const MASK_LEDGER_HEADER_FLAGS = 0x7;
MASK_LEDGER_HEADER_FLAGS: int = 0x7
#: const AUTH_MSG_FLAG_FLOW_CONTROL_BYTES_REQUESTED = 200;
AUTH_MSG_FLAG_FLOW_CONTROL_BYTES_REQUESTED: int = 200
#: const TX_ADVERT_VECTOR_MAX_SIZE = 1000;
TX_ADVERT_VECTOR_MAX_SIZE: int = 1000
#: const TX_DEMAND_VECTOR_MAX_SIZE = 1000;
TX_DEMAND_VECTOR_MAX_SIZE: int = 1000
#: const MAX_OPS_PER_TX = 100;
MAX_OPS_PER_TX: int = 100
#: const LIQUIDITY_POOL_FEE_V18 = 30;
LIQUIDITY_POOL_FEE_V18: int = 30
#: const SC_SPEC_DOC_LIMIT = 1024;
SC_SPEC_DOC_LIMIT: int = 1024
#: const SCSYMBOL_LIMIT = 32;
SCSYMBOL_LIMIT: int = 32
#: const CONTRACT_COST_COUNT_LIMIT = 1024;
CONTRACT_COST_COUNT_LIMIT: int = 1024
